function [R, D] = compute_rand(c1, c2)
% compute_rand - compute the Rand index between two clusterings
%
%    [R, D] = compute_rand(c1, c2);
%
%  -- INPUT --
%    c1 = vector (or matrix) containing first cluster assignments 
%    c2 = vector (or matrix) containing second cluster assignments
%
%    Note: the cluster labels are 1,2,...
%
%  -- OUTPUT --
%    R = Rand index (probability of agreement)
%    D = probability of disagreement
%

c1 = c1(:);
c2 = c2(:);

if any(c1==0) || any(c2==0)
  error('found class zero!');
end

K1 = max(c1);
K2 = max(c2);
  
n = length(c1);
  
mtx = zeros(K1, K2);
for i=1:K1
  for j=1:K2
    % count aggreement between two classes
    mtx(i,j) = sum((c1==i).*(c2==j));
  end
end

[R D] = clusterrand(mtx);
  



function [A, D] = clusterrand(mtx)
% clusterrand - compute cluster Rand number from agreement matrix
%
%     [A, D] = clusterrand(mtx)
%   
%    A = agreements percent
%    D = disagreements percent
%

% sum rows and columns
mtxi = sum(mtx,2);  % sum over rows
mtxj = sum(mtx,1);  % sum over columns
n = sum(sum(mtx));

nk2 = nchoosek(n,2);

A = nk2 + mtx(:)'*mtx(:) - 0.5*(mtxi'*mtxi + mtxj*mtxj');
D = 0.5*(mtxi'*mtxi + mtxj*mtxj') - mtx(:)'*mtx(:);

A = A / nk2;
D = D / nk2;

